#include "sentence.h"

FUN int GenerateVerificationCond()
{
	ProcedureListPtr pl = currentProcedureList;

	while( pl->procedureList != NULL ) {
		Verification( pl->procedure );
		pl = pl->procedureList;
        }
	currentProcedureList = pl;
}

FUN int Verification( proc )
ProcedurePtr proc;
{
	FormulaListPtr pl;
	FormulaPtr post;
	FormulaPtr prepost;;

	if (proc->body==NULL || *(proc->body)==0) return;
	currentProcedure = proc;

	pl = Formula2FormulaList( proc->postCond, LAND );
	pl = FormulaListSelect( pl, proc, isNotEgenVar );
	post = FormulaList2Formula( pl, LAND );

	AddPostCond( proc->body, post );
	MakeAssumption( proc->preCond, CalPreCond( proc->body ));
}

FUN FormulaPtr CalPreCond( label )
char *label;
{
	SentencePtr s = Sentence2Ptr( label );

	switch (s->type) {
		case ELEM:
			return( CalElem( s ));
			break;
		case COND:
			return( CalCase( s ));
			break;
		case LOOP:
			return( CalLoop( s ));
			break;
		case SEQ:
			return( CalSeq( s ));
			break;
		case CALL:
			return( CalCall( s ));
			break;
	}
}

/*------------------------------------------------------------*/

FUN int MakeAssumption( c1, c2 )
FormulaPtr c1, c2;
{
	AssumptionListPtr ap = &assumptionList;

	while (ap->assumption != NULL) ap=ap->assumptionList;

	ap->assumption = (AssumptionPtr) calloc( sizeof( Assumption ), 1 );
	ap->assumption->formula1 = FormulaCopy( c1 ); 
	ap->assumption->formula2 = FormulaCopy( c2 ); 
        strcpy( ap->assumption->procName, currentProcedure->name );
	
	ap->assumptionList = 
		(AssumptionListPtr) calloc( sizeof( AssumptionList ), 1 );
}

FUN int MakeLoopAssumption( c, c1, c2 )
FormulaPtr c, c1, c2;
{
	AssumptionListPtr ap = &assumptionList;

	while (ap->assumption != NULL) ap=ap->assumptionList;

	ap->assumption = (AssumptionPtr) calloc( sizeof( Assumption ), 1 );
	ap->assumption->formula1 = CombineCond( FormulaCopy( c ), '^', FormulaCopy( c1 ) );
	ap->assumption->formula2 = FormulaCopy( c2 );
        strcpy( ap->assumption->procName, currentProcedure->name );
	
	ap->assumptionList = 
		(AssumptionListPtr) calloc( sizeof( AssumptionList ), 1 );
}
 
FUN FormulaPtr CombineCaseCond( c1, c2, c )
FormulaPtr c, c1, c2;
{
        FormulaPtr t1, t2, t3;
 
        t1 = CombineCond( NULL, '~', c );
        t2 = CombineCond( t1, '|', c1 );
        t3 = CombineCond( c, '|', c2 );
        t1 = CombineCond( t2, '^', t3 );
        return( t1 );
}

/*---------------------------------------------------------------------*/

FUN int AssumptionPrint( pname, ss )
char *pname, *ss;
{
	char s1[ MOPROG ];
	AssumptionListPtr p = &assumptionList;

	while( p->assumption != NULL ) {
		if (strcmp( p->assumption->procName, pname )==0 ) {
			FormulaPrint( p->assumption->formula1, s1 );
			NiceFormula( s1, ss, 40 );
			sprintf( ss+strlen(ss), "\n =>\n  " );
			ss+=strlen(ss);
			FormulaPrint( p->assumption->formula2, s1 );
			NiceFormula( s1, ss, 40 );
			sprintf( ss+strlen(ss), "\n\n" );
			ss+=strlen(ss);
		}
		p = p->assumptionList;
        }
}
 
FUN int AssumptionListPrint( ss )
char *ss;
{
	ProcedureListPtr pl = &procedureList;

	*ss=0;
	while( pl->procedureList != NULL ) {
                sprintf( ss, "...for %s...\n\n", pl->procedure->name );
                AssumptionPrint( pl->procedure->name, ss+strlen(ss) ); 
		strcat( ss, "\n" );
		ss += strlen( ss );
		pl = pl->procedureList;
        }
}

/*----------------------------------------------------*/

