#include "term.h"

extern AssignListPtr AvoidBoundVarSelect();
/*------------------------------------------------------------------*/

FUN int FormulaFree( t )
FormulaPtr t; 
{
	if (t==NULL) return( NULL );

	if (t->connective==LALL) {
		TermFree( t->left );
		FormulaFree( t->right );
	} else if (isLog(t->connective)) {
		FormulaFree( t->left );
		FormulaFree( t->right );
	} else if (isRelPred(t->connective)) {
		TermFree( t->left );
		TermFree( t->right );
	} 
	free( t );
}

FUN FormulaPtr FormulaCopy( t )
FormulaPtr t; 
{
	FormulaPtr t1; 

	if (t==NULL) return( NULL );

	t1 = (FormulaPtr) calloc( sizeof( Formula ), 1 );
	t1->connective = t->connective;

	if (t->connective==LALL) {
		t1->left = (Ptr) TermCopy( t->left );
		t1->right = (Ptr) FormulaCopy( t->right );
	} else if (isLog(t->connective)) {
		t1->left = (Ptr) FormulaCopy( t->left );
		t1->right = (Ptr) FormulaCopy( t->right );
	} else if (isRelPred(t->connective)) {
		t1->left = (Ptr) TermCopy( t->left );
		t1->right = (Ptr) TermCopy( t->right );
	}
	
	return( t1 );
}

FUN void FormulaMove( t1, t2 )
FormulaPtr t1, t2;
{
	t2->left = t1->left;
	t2->right = t1->right;
	t2->connective  = t1->connective;
}

FUN FormulaPtr CombineCond( c1, c, c2 )
char c;
FormulaPtr c1, c2;
{
        FormulaPtr t;
 
        t = (FormulaPtr) calloc( sizeof( Formula ), 1 );
        t->connective = c;
        t->left = (Ptr) c1;
        t->right = (Ptr) c2;
        return( t );
}
 
/*------------------------------------------------------------------*/


FUN FormulaPtr FormulaSubstSim( t, sl )
FormulaPtr t; AssignListPtr sl;
{
	if (t==NULL) return( NULL );

	if (t->connective==LALL) {
		sl = AvoidBoundVarSelect( sl, t->left );
		t->right = (Ptr) FormulaSubstSim( t->right, sl );
/*
		CleanPtrList( sl );
*/
	} else if (isLog(t->connective)) {
		t->left = (Ptr) FormulaSubstSim( t->left, sl );
		t->right = (Ptr) FormulaSubstSim( t->right, sl );
	} else if (isRelPred(t->connective)) {
		t->left = (Ptr) TermSubstSim( t->left, sl );
		t->right = (Ptr) TermSubstSim( t->right, sl );
	}
	
	return( t );
}

/*------------------------------------------------------------------*/


FUN int FormulaEqual( p1, p2 )
FormulaPtr p1, p2;
{
        if (p1==NULL || p2==NULL) {
        	if (p1==NULL && p2==NULL) return( 1 );
		else return( 0 );
	}
	if ( p1->connective != p2->connective ) return( 0 );

        if (p1->connective==LALL) {
		if (!TermEqual( p1->left, p2->left )) return( 0 );
		if (!FormulaEqual( p1->right, p2->right )) return( 0 );
        } else if (isLog(p1->connective)) {
		if (!FormulaEqual( p1->left, p2->left )) return( 0 );
		if (!FormulaEqual( p1->right, p2->right )) return( 0 );
        } else if (p1->connective=='=') {
		if (!TermEqual( p1->left, p2->left )) {
			if (!TermEqual( p1->left, p2->right )) return( 0 );
			if (!TermEqual( p1->right, p2->left )) return( 0 );
			return( 1 );
		}
		if (!TermEqual( p1->right, p2->right )) return( 0 );
        } else if (isRelPred(p1->connective)) {
		if (!TermEqual( p1->left, p2->left )) return( 0 );
		if (!TermEqual( p1->right, p2->right )) return( 0 );
        } 

        return( 1 );
}

/*------------------------------------------------------------*/

FUN AssignListPtr AvoidBoundVarSelect( p, v )
AssignListPtr p; TermPtr v;
{
        AssignListPtr c = p;
        AssignListPtr c1, c2, c3;
        int first = 1;
 
        c3 = c2 = c1 = 
                (AssignListPtr) calloc( sizeof( AssignList ), 1 );
        while (c != NULL ) {  
                if (!VarEqual( c->assign->termLeft, v)) {
                        c1->assign = c->assign;
                        c1->assignList = (AssignListPtr)
                                calloc( sizeof( AssignList ), 1 );
                        c2 = c1;
                        c1 = c1->assignList;
                }
                c = c->assignList;
        }
        if (c2->assignList != NULL) {
                free( c2->assignList );
                c2->assignList = NULL;
        } else {
                free( c3 );
                c3 = NULL;
        }
        return( c3 );
}
 
/*------------------------------------------------------------*/

/*

FUN int FormulaReadOld( s, t )
char* s;
FormulaPtr t;
{
	char *ss=s;

	ss += FormulaRead1( ss, t );
	while (*ss) {
		if (*ss==LAND) {
			FormulaPtr t1 = (FormulaPtr) calloc( sizeof( Formula ), 1 );
			FormulaPtr t2 = (FormulaPtr) calloc( sizeof( Formula ), 1 );
			ss += FormulaRead1( ss+1, t2 ) + 1;
			FormulaMove( t, t1 );
			t->left = (Ptr) t1;
			t->right = (Ptr) t2;
			t->connective = LAND;
		} else if (*ss==LOR) {
			FormulaPtr t1 = (FormulaPtr) calloc( sizeof( Formula ), 1 );
			FormulaPtr t2 = (FormulaPtr) calloc( sizeof( Formula ), 1 );
			ss += FormulaRead( ss+1, t2 ) + 1;
			FormulaMove( t, t1 );
			t->left = (Ptr) t1;
			t->right = (Ptr) t2;
			t->connective = LOR;
		} else if (*ss==IMP1) {
			FormulaPtr t1 = (FormulaPtr) calloc( sizeof( Formula ), 1 );
			FormulaPtr t2 = (FormulaPtr) calloc( sizeof( Formula ), 1 );
			if (*(ss+1)==IMP2) ss++; else Error( "formula read, imp" );
			ss += FormulaRead( ss+1, t2 ) + 1;
			FormulaMove( t, t1 );
			t->left = (Ptr) t1;
			t->right = (Ptr) t2;
			t->connective = IMP;
		} else {
			break;
		}
	}
	return( ss - s );
}

*/
			
