#include "term.h"

FUN int FormulaListFree( p )
FormulaListPtr p;
{
        if (p==NULL) return;
        FormulaListFree( p->formulaList );

	FormulaFree( p->formula );

        free( p );
}

FUN FormulaListPtr FormulaListCopy( p )
FormulaListPtr p;
{
	FormulaListPtr c;

        if (p==NULL) return( NULL );
	c = (FormulaListPtr) calloc( sizeof( FormulaList ), 1 );
	c->formula = FormulaCopy( p->formula );
        c->formulaList = FormulaListCopy( p->formulaList );
	return( c );
}

/*---------------------------------------------------*/

FUN FormulaListPtr FormulaListAppend( c1, c2 )
FormulaListPtr c1, c2;
{
	FormulaListPtr c = c1;

	if (c==NULL) return( c2 );
	while (c->formulaList != NULL ) c = c->formulaList;
	c->formulaList = c2;
	return( c1 );
}

FUN FormulaListPtr Formula2FormulaList( p, ch )
FormulaPtr p; char ch;
{
	FormulaListPtr q;

	if (p==NULL) return( NULL );
	if (p->connective == ch) {
		return( FormulaListAppend( 
			Formula2FormulaList( p->left, ch ),
			Formula2FormulaList( p->right, ch )));
	} else {
		q = (FormulaListPtr) calloc( sizeof( FormulaList ), 1 );
		q->formula = p;
		return( q );
	}
}

FUN FormulaPtr FormulaList2Formula( a, ch )
FormulaListPtr a; char ch;
{
	FormulaPtr b, c;

	if (a==NULL) return( NULL );
	b = a->formula;
	a = a->formulaList;

	while( a!=NULL) {
		c = (FormulaPtr) calloc( sizeof( Formula ), 1 );
		c->connective = ch;
		c->left = (Ptr) b;
		c->right = (Ptr) (a->formula);
		b = c;
		a = a->formulaList;
	}
	return( b );
}

FUN AssignListPtr FormulaList2AssignList( p )
FormulaListPtr p;
{
	AssignListPtr q;

	if (p==NULL) return( NULL );

	q = (AssignListPtr) calloc( sizeof( AssignList ), 1 );
	q->assign = (AssignPtr) calloc( sizeof( Assign ), 1 );

	if (p->formula->connective!='=') Error( "FormulaList2Assign" );
	q->assign->termLeft = (TermPtr) p->formula->left;
	q->assign->termRight = (TermPtr) p->formula->right;
	q->assignList =  FormulaList2AssignList( p->formulaList );
	return( q );
}

/*------------------------------------------------------*/

FUN FormulaListPtr FormulaListSubstSim( a, s )
FormulaListPtr a;
AssignListPtr s;
{
	FormulaListPtr b=a;
	if ( b==NULL) return( NULL );

	while( b->formula!=NULL) {
		b->formula = FormulaSubstSim( b->formula, s );
		if (b->formulaList == NULL ) break;
		else b = b->formulaList;
	}
	return( a );
}

FUN FormulaListPtr FormulaListSubstSim1( p, a, s )
FormulaPtr p;
FormulaListPtr a;
AssignListPtr s;
{
	FormulaListPtr b=a;
	if ( b==NULL) return( NULL );

	while( b->formula!=NULL) {
		if (p!=b->formula) b->formula = FormulaSubstSim( b->formula, s );

		if (b->formulaList==NULL) break;
		b = b->formulaList;
	}
	return( a );
}

FUN FormulaListPtr FormulaListRewrite( a )
FormulaListPtr a;
{
	FormulaListPtr b=a;
	AssignList c;
	Assign c1;

	if (b==NULL) return( NULL );

	while( b->formula!=NULL) {
		if (isAssign1( b->formula )) {
			c1.termLeft = (TermPtr) b->formula->left;
			c1.termRight = (TermPtr) b->formula->right;
			c.assign = &c1;
			c.assignList = NULL;
			FormulaListSubstSim1( b->formula, a, &c );
		}
		if (b->formulaList==NULL) break;
		b = b->formulaList;
	}
	return( a );
}

/*---------------------------------------------------------------------*/

FUN int FormulaListPrint( p, ss )
FormulaListPtr p; char *ss;
{

	*ss=0;
	while( p!=NULL) {
		FormulaPrint( p->formula, ss );
		if( p->formulaList==NULL ) break;
		sprintf( ss+strlen(ss), "," );
		ss += strlen( ss );
		p = p->formulaList;
	}
}

FUN int CleanPtrList( p )
FormulaListPtr p;
{
        if (p==NULL) return;
        CleanPtrList( p->formulaList );
        free( p );
}

/*------------------------------------------------------------------*/

FUN int isAssign1( p )
FormulaPtr p;
{
        Ptr p0;
        TermPtr left, right;
 
        if (p->connective == '=') {
                left = (TermPtr) p->left;
                right = (TermPtr) p->right;

		if (left->type == VAR && right->type == VAR) {
			if (isdigit(*(left->name))) {
 				if (isdigit(*(right->name))) return( 0 );
                        	p0 = p->left;
                        	p->left = p->right;
                        	p->right = p0;
                        	return( 1 );
			} else return( 1 );
		}

                if ( left->type == VAR ) {
			if ( !isConst( left->name ) ||
				(isEconst(left->name) && (right->type==FUNC))) 
			return( 1 );
		}
				
                if ( right->type == VAR ) {
			if ( !isConst( right->name ) ||
				(isEconst(right->name) && (left->type==FUNC))) {
                        	p0 = p->left;
                        	p->left = p->right;
                        	p->right = p0;
                        	return( 1 );
			}
                }
        }
        return( 0 );
}

