#include "proof.h"
 

/*-------------------------------------------------------*/


FUN int SimplifySequentUseRules( s )
SequentPtr s;
{
	/*Logical Rules*/
	FormulaListPtr fl1, fl2;
	FormulaPtr p1, p2;
	SequentPtr newSeq;
	int k=0;

	if (s==NULL) return( 1 );

	SimplifySequentUseAxioms( s );

	fl1 = s->formulaList1;
	fl2 = s->formulaList2;


	while( fl1 != NULL ) {
		p1  = fl1->formula;
		if (p1==NULL || !isCon(p1->connective)) {
			fl1=fl1->formulaList;
			continue;
		}
		k = 1;
		s->formulaList1 = SetRemove( p1, s->formulaList1 );
		switch (p1->connective) {
			case LNEG:
				s->formulaList2 = SetAdd( 
					p1->right, 
					s->formulaList2 );
				break;
			case LAND:
				s->formulaList1 = FormulaListAppend( 
					Formula2FormulaList( p1, LAND ), 
					s->formulaList1 );
				break;
			case IMP:
				p2 = (FormulaPtr) p1->left;
				p1 = (FormulaPtr) p1->right;
				newSeq = NewSequent( s  );
				newSeq->formulaList1 = SetAdd( 
					p2, 
					newSeq->formulaList1 );
				s->formulaList2 = SetAdd( p1, s->formulaList2 );
				break;
			case LOR:
				p2 = (FormulaPtr) p1->left;
				p1 = (FormulaPtr) p1->right;
				newSeq = NewSequent( s  );
				newSeq->formulaList1 = SetAdd( 
					p2, 
					newSeq->formulaList1 );
				s->formulaList1 = SetAdd( p1, s->formulaList1 );
				break;
			default:
				Error( "sequent" );
		}
		fl1=fl1->formulaList;
	}
	
	while( fl2 != NULL ) {
		p1  = fl2->formula;
		if (p1==NULL || !isCon(p1->connective)) {
			fl2=fl2->formulaList;
			continue;
		}
		k = 1;

		s->formulaList2 = SetRemove( p1, s->formulaList2 );
		switch (p1->connective) {
			case LNEG:
				s->formulaList1 = SetAdd( 
					p1->right, 
					s->formulaList1 );
				break;
			case IMP:
				p2 = (FormulaPtr) p1->left;
				p1 = (FormulaPtr) p1->right;
				s->formulaList1 = SetAdd( p1, s->formulaList1 );
				s->formulaList2 = SetAdd( p2, s->formulaList2 );
				break;
			case LOR:
				s->formulaList2 = FormulaListAppend( 
					Formula2FormulaList( p1, LOR ), 
					s->formulaList2 );
				break;
			case LAND:
				p2 = (FormulaPtr) p1->left;
				p1 = (FormulaPtr) p1->right;
				newSeq = NewSequent( s  );
				newSeq->formulaList2 = SetAdd( 
					p2, 
					newSeq->formulaList2 );
				s->formulaList2 = SetAdd( p1, s->formulaList2 );
				break;
			default:
				Error( "sequent" );
		}
		fl2=fl2->formulaList;
	}
	if (k) SimplifySequentUseRules( s );

}

/*-------------------------------------------------------*/

FUN int SimplifySequentUseAxioms( s )
SequentPtr s;
{
	FormulaListPtr fl1, fl2;
	FormulaPtr p1;
	int ok=0;

	if (s==NULL) return( 1 );

	fl2 = s->formulaList2;

	while( fl2 != NULL ) {
		p1  = fl2->formula;
		if (p1==NULL) {
			fl2 = fl2->formulaList;
			continue;
		}

		ok = isTrue( p1 ) || SetInclude( p1, s->formulaList1 );
		if (ok) {
			FormulaListFree( s->formulaList1 ); 
			s->formulaList1 = NULL;
			FormulaListFree( s->formulaList2 ); 
			s->formulaList2 = NULL;
			return;
		} else if (isFalse(p1)) {
			s->formulaList2 = SetRemove( p1, s->formulaList2 );
		}

		fl2 = fl2->formulaList;
	}


	fl1 = s->formulaList1;

	while( fl1 != NULL ) {
		p1  = fl1->formula;
		if (p1==NULL) {
			fl1 = fl1->formulaList;
			continue;
		}
		if (isFalse(p1)) {
			FormulaListFree( s->formulaList1 ); 
			s->formulaList1 = NULL;
			FormulaListFree( s->formulaList2 ); 
			s->formulaList2 = NULL;
			return;
		} else if (isTrue(p1)) {
			s->formulaList1 = SetRemove( p1, s->formulaList1 );
		}

		fl1 = fl1->formulaList;
	}


}

/*----------------------------------------------------------*/


FUN int isTrue( p )
FormulaPtr p;
{
	if (p->connective=='=') {
		if (TermEqual( p->left, p->right )) {
			p->connective=LTRUE;
			p->left=NULL;
			p->right=NULL;
		}
	}

	return(p->connective==LTRUE);
}

FUN int isFalse( p )
FormulaPtr p;
{
	if (p->connective=='>') {
		if (TermEqual( p->left, p->right )) {
			p->connective = LTRUE;
			p->left = NULL;
			p->right = NULL;
			p->right = (Ptr) FormulaCopy( p );
			p->connective = LNEG;
		}
	}

	if (p->connective==LNEG) return(isTrue( p->right ));
	else return( 0 );
}

/*----------------------------------------------------------*/
