#include "sentence.h"

extern SentencePtr Sentence2Ptr();
extern void DivideLabels();
extern FormulaPtr GetPostCond();

FUN int SentenceRead( s, l )
char* s;
char *l;
{
	char l1[ MOLABEL ];
	char l2[ MOLABEL ];
	char l3[ MOLABEL ];
	FormulaPtr c;
	int first = 1;

	*l = 0;
	if ( *s == '[' ) { s++; }
	while (*s ) {
		if ( *s == ']' ) break;
		if ( isLoop( s ) ) s += SentLoopRead( s, l );
		else if ( isCase( s ) ) s += SentCaseRead( s, l );
		else if ( isElem( s ) ) s += SentElemRead( s, l );
		else if ( isCall( s ) ) s += SentCallRead( s, l );
		else if ( isInv( s ) ) {
			c = (FormulaPtr) calloc( sizeof( Formula ), 1 );
			s += AssertionRead( s, c );

			AddPostCond( l3, c );

			continue;
		} 
		else { Error( "sentenceRead" ); Error( s ); break; }
		if (first) {
			strcpy( l3, l );
			DivideLabels( l, l1, l2 );
			first = 0;
		} else { 
			SentSeqComposition( l3, l );
			strcpy( l3, l );
		}
	}
}

/*---------------------------------------------------------------------*/

FUN SentencePrint( l, s, k )
char *l; char *s;
{
	*s = 0;
	if (l==NULL || *l==0) return;
	SentencePrintPtr( Sentence2Ptr( l ), s, k );
}

FUN SentencePrintPtr( p, s, k )
SentencePtr p;
char *s;
int k;
{
	char s0[ MOLABEL ];
	char s1[ MOLABEL ];
	char s2[ MOLABEL ];
	char s3[ MOLABEL ];
	char t1[ MOPROG ];
	char t2[ MOPROG ];
	char *ss;

	if (p==NULL) return;

	if (p->type!=SEQ) Blanks( k, s );
	ss = s+strlen( s );

	switch (p->type) {
		case SEQ:
			SentencePrint( p->labels1, t1, k );
			SentencePrint( p->labels2, t2, k );
			sprintf( ss, "%s%s", t1, t2 );
			break;
		case COND:
			DivideLabels( p->labels, s0, s2 );
			DivideLabels( p->labels1, s1, s2 );
			DivideLabels( p->labels2, s3, s2 );
			FormulaPrint( p->cond, t1 );
			sprintf( ss, "?[LB=%s^%s", s0, t1 );
			ss += strlen( ss );
			SentencePrint( p->labels1, t1, k+2 );
			SentencePrint( p->labels2, t2, k+2 );
			sprintf( ss, "=>$OLB=%s|~=>$OLB=%s;\n%s%s", 
				s1, s3, t1, t2 );
			Blanks( k, ss + strlen( ss ));
			ss += strlen( ss );
			sprintf( ss, " ]\n" );
			break;
		case ELEM:
        		DivideLabels( p->labels, s1, s2 );
        		sprintf(ss, "LB=%s=>", s1 );
        		ss += strlen( ss );
        		AssignListPrint( p->assignList, ss );
        		ss += strlen( ss );
        		sprintf(ss, "^$OLB=%s;\n", s2 );
			break;
		case LOOP:
			DivideLabels( p->labels, s1, s2 );
			DivideLabels( p->labels1, s3, s1 );
			FormulaPrint( p->cond, t1 );
			sprintf( ss, "*[LB=%s^%s", s1,t1 );
			ss += strlen( ss );
			SentencePrint( p->labels1, t1, k+2 );
			sprintf( ss, "=>($OLB=%s|$OLB=%s)\n%s", s3, s2, t1 );
			ss += strlen( ss ); 
			FormulaPrint( GetPostCond( p->labels1 ), t1 );
			Blanks( k+2, ss + strlen( ss ));
			ss += strlen( ss );
			sprintf( ss, "{%s}\n", t1 );
			Blanks( k, ss + strlen( ss ));
			ss += strlen( ss );
			sprintf( ss, " ]\n", t1 );
			break;
		case CALL:
			DivideLabels( p->labels, s1, s2 );
        		sprintf(ss, "LB=%s=>%s(", s1, p->labels1 );
        		ss += strlen( ss );
			SentCallParamPrint( p->labels1, p->assignList, ss );
        		ss += strlen( ss );
        		sprintf(ss, ")^$OLB=%s;\n", s2 );
			break;
	}
}
			
/*-----------------------------*/

FUN SentCallParamPrint( pname, tl, s )
char *pname; 
AssignListPtr tl;
char *s;
{
        ProcedureListPtr p = &procedureList;
        ParamTrippleListPtr p1;
 
        while ( p!=NULL ) {
                if (strcmp( p->procedure->name, pname )==0) {
			p1 = p->procedure->params;
			break;
                }
        	p = p->procedureList;
        }
	if (p==NULL) return;

	while (p1 != NULL ) {
                if (p1->paramTrippleList==NULL ) break;
		sprintf( s+strlen(s), "%s/", p1->io );
		TermPrint( tl->assign->termRight, s+strlen(s) );
		sprintf( s+strlen(s), "|" );
		TermPrint( tl->assign->termLeft, s+strlen(s) );
		sprintf( s+strlen(s), ";" );
		p1 = p1->paramTrippleList;
		tl = tl->assignList;
	}
	if (p1 != NULL ) {
		sprintf( s+strlen(s), "%s/", p1->io );
		TermPrint( tl->assign->termRight, s+strlen(s) );
		sprintf( s+strlen(s), "|" );
		TermPrint( tl->assign->termLeft, s+strlen(s) );
	}

}

/*--------------------------------------------------------------*/

FUN int LabelRead( s, l )
char* s;
char* l;
{
	int i;

	if ( isLB( s ) ) {
		i = 3;
		while ( s[i]!=';' && !(isPdel( s[ i ] )) ) *l++ = s[ i++ ];
		*l=0;
	} else Error( "LabelRead" );
	return( i );
}



FUN SentencePtr Sentence2Ptr( l )
char *l;
{
        SentenceListPtr p = currentProcedure->sentenceList;
 
        while ( p!=NULL ) {
                if (strcmp( p->sentence->labels, l )==0) {
                        return( p->sentence );
                }
        	p = p->sentenceList;
        }
        return( NULL );
}        
 
FUN void DivideLabels( s, s1, s2 )
char *s, *s1, *s2;
{
        char *t, *t1, *t2;
        t=s; t1=s1; t2=s2;
        s++;
        while ( *s != ',' ) *s1++=*s++;
        *s1 = 0;
        s++;
        while ( *s != ')' ) *s2++=*s++;
        *s2 = 0;
}

FUN Blanks( k, s )
int k; char *s;
{
	int i=k;

	*s = 0;
	while (i--) sprintf( s+strlen(s), "%c", ' ' );
}

/*----------------------------------------------------------------*/

FUN FormulaPtr GetPostCond( l )
char *l;
{
	SentenceListPtr p1 = currentProcedure->sentenceList;
        ProcedureListPtr p2 = &procedureList;

	while ( p1!=NULL ) {
                if (strcmp( p1->sentence->labels, l )==0) {
                        return( p1->sentence->postCond );
                }
        	p1 = p1->sentenceList;
        }

        while ( p2!=NULL ) {
                if (strcmp( p2->procedure->name, l )==0) {
                        return( p2->procedure->postCond );
                }
        	p2 = p2->procedureList;
        }

	return( NULL );
}

FUN int AddPreCond( l, p )
char *l;
FormulaPtr p;
{
        ProcedureListPtr p2 = &procedureList;

        while ( p2!=NULL ) {
                if (strcmp( p2->procedure->name, l )==0) {
                        p2->procedure->preCond = p;
                        return( 1 );
                }
        	p2 = p2->procedureList;
        }

	return( 0 );
}

FUN int AddPostCond( l, p )
char *l;
FormulaPtr p;
{
	SentenceListPtr p1 = currentProcedure->sentenceList;
        ProcedureListPtr p2 = &procedureList;

	while ( p1->sentence!=NULL ) {
                if (strcmp( p1->sentence->labels, l )==0) {
                        p1->sentence->postCond = p;
                        return( 1 );
                }
        	p1 = p1->sentenceList;
        }

        while ( p2->procedure!=NULL ) {
                if (strcmp( p2->procedure->name, l )==0) {
                        p2->procedure->postCond = p;
                        return( 1 );
                }
        	p2 = p2->procedureList;
        }

	return( 0 );
}

/*---------------------------------------------------------------------*/

FUN int isLoop( s )
char *s;
{
	return( *s == '*' && *(s+1) == '[' );
}

FUN int isCase( s )
char *s;
{
	return( *s == '?' && *(s+1) == '[' );
}

FUN int isInv( s )
char *s;
{
	return( *s == '{' );
}

FUN int isElem( s )
char *s;
{
	if ( isLB( s ) ) s += 3; else return( 0 );
	while ( *s && !isIMP( s ) ) s++;
	s += 2;
	if ( isNEXT( s ) ) return( 1 );
	else return( 0 );
}

FUN int isCall( s )
char *s;
{
	if ( isLB( s ) ) s += 3; else return( 0 );

	while ( *s && !isIMP( s ) ) s++;
	s += 2;
	if ( ! isNEXT( s ) ) return( 1 );
	else return( 0 );
}

